@@ -1,5 +1,5 @@
 #include <dlfcn.h>
-#include <utils.h>
+#include <blockdev/utils.h>
 #include "blockdev.h"
 #include "plugins.h"
 
@@ -13,14 +13,18 @@
 #include "plugin_apis/loop.c"
 #include "plugin_apis/crypto.h"
 #include "plugin_apis/crypto.c"
-#include "plugin_apis/mpath.c"
 #include "plugin_apis/mpath.h"
-#include "plugin_apis/dm.c"
+#include "plugin_apis/mpath.c"
 #include "plugin_apis/dm.h"
+#include "plugin_apis/dm.c"
 #include "plugin_apis/mdraid.h"
 #include "plugin_apis/mdraid.c"
 #include "plugin_apis/kbd.h"
 #include "plugin_apis/kbd.c"
+#include "plugin_apis/part.h"
+#include "plugin_apis/part.c"
+#include "plugin_apis/fs.h"
+#include "plugin_apis/fs.c"
 
 #if defined(__s390__) || defined(__s390x__)
 #include "plugin_apis/s390.h"
@@ -39,21 +43,23 @@
 
 static GMutex init_lock;
 static gboolean initialized = FALSE;
+static GMutex env_lock;
 
 typedef struct BDPluginStatus {
     BDPluginSpec spec;
     gpointer handle;
 } BDPluginStatus;
 
-typedef void* (*LoadFunc) (gchar *so_name);
+typedef void* (*LoadFunc) (const gchar *so_name);
 
 /* KEEP THE ORDERING OF THESE ARRAYS MATCHING THE BDPluginName ENUM! */
 static gchar * default_plugin_so[BD_PLUGIN_UNDEF] = {
-    "libbd_lvm.so.""0", "libbd_btrfs.so.""0",
-    "libbd_swap.so.""0", "libbd_loop.so.""0",
-    "libbd_crypto.so.""0", "libbd_mpath.so.""0",
-    "libbd_dm.so.""0", "libbd_mdraid.so.""0",
-    "libbd_kbd.so.""0","libbd_s390.so.""0"
+    "libbd_lvm.so.""2", "libbd_btrfs.so.""2",
+    "libbd_swap.so.""2", "libbd_loop.so.""2",
+    "libbd_crypto.so.""2", "libbd_mpath.so.""2",
+    "libbd_dm.so.""2", "libbd_mdraid.so.""2",
+    "libbd_kbd.so.""2","libbd_s390.so.""2",
+    "libbd_part.so.""2", "libbd_fs.so.""2"
 };
 static BDPluginStatus plugins[BD_PLUGIN_UNDEF] = {
     {{BD_PLUGIN_LVM, NULL}, NULL},
@@ -65,13 +71,15 @@ static BDPluginStatus plugins[BD_PLUGIN_UNDEF] = {
     {{BD_PLUGIN_DM, NULL}, NULL},
     {{BD_PLUGIN_MDRAID, NULL}, NULL},
     {{BD_PLUGIN_KBD, NULL}, NULL},
-    {{BD_PLUGIN_S390, NULL}, NULL}
+    {{BD_PLUGIN_S390, NULL}, NULL},
+    {{BD_PLUGIN_PART, NULL}, NULL},
+    {{BD_PLUGIN_FS, NULL}, NULL},
 };
 static gchar* plugin_names[BD_PLUGIN_UNDEF] = {
-    "lvm", "btrfs", "swap", "loop", "crypto", "mpath", "dm", "mdraid", "kbd", "s390"
+    "lvm", "btrfs", "swap", "loop", "crypto", "mpath", "dm", "mdraid", "kbd", "s390", "part", "fs"
 };
 
-static void set_plugin_so_name (BDPlugin name, gchar *so_name) {
+static void set_plugin_so_name (BDPlugin name, const gchar *so_name) {
     plugins[name].spec.so_name = so_name;
 }
 
@@ -116,7 +124,7 @@ static GSequence* get_config_files (GError **error) {
     return ret;
 }
 
-static gboolean process_config_file (gchar *config_file, GSList **plugins_sonames, GError **error) {
+static gboolean process_config_file (const gchar *config_file, GSList **plugins_sonames, GError **error) {
     GKeyFile *config = NULL;
     BDPlugin i = 0;
     gchar **sonames = NULL;
@@ -212,6 +220,13 @@ static void unload_plugins () {
     plugins[BD_PLUGIN_S390].handle = NULL;
 #endif
 
+    if (plugins[BD_PLUGIN_PART].handle && !unload_part (plugins[BD_PLUGIN_PART].handle))
+        g_warning ("Failed to close the part plugin");
+    plugins[BD_PLUGIN_PART].handle = NULL;
+
+    if (plugins[BD_PLUGIN_FS].handle && !unload_fs (plugins[BD_PLUGIN_FS].handle))
+        g_warning ("Failed to close the fs plugin");
+    plugins[BD_PLUGIN_FS].handle = NULL;
 }
 
 static void load_plugin_from_sonames (BDPlugin plugin, LoadFunc load_fn, void **handle, GSList *sonames) {
@@ -246,6 +261,10 @@ static void do_load (GSList **plugins_sonames) {
     if (!plugins[BD_PLUGIN_S390].handle && plugins_sonames[BD_PLUGIN_S390])
         load_plugin_from_sonames (BD_PLUGIN_S390, load_s390_from_plugin, &(plugins[BD_PLUGIN_S390].handle), plugins_sonames[BD_PLUGIN_S390]);
 #endif
+    if (!plugins[BD_PLUGIN_PART].handle && plugins_sonames[BD_PLUGIN_PART])
+        load_plugin_from_sonames (BD_PLUGIN_PART, load_part_from_plugin, &(plugins[BD_PLUGIN_PART].handle), plugins_sonames[BD_PLUGIN_PART]);
+    if (!plugins[BD_PLUGIN_FS].handle && plugins_sonames[BD_PLUGIN_FS])
+        load_plugin_from_sonames (BD_PLUGIN_FS, load_fs_from_plugin, &(plugins[BD_PLUGIN_FS].handle), plugins_sonames[BD_PLUGIN_FS]);
 }
 
 static gboolean load_plugins (BDPluginSpec **require_plugins, gboolean reload, guint64 *num_loaded) {
@@ -357,6 +376,20 @@ GQuark bd_init_error_quark ()
  *
  * Returns: whether the library was successfully initialized with all the
  *          required or default (see @require_plugins) plugins or not
+ *
+ * Example of libblockdev initialization with 'fs' and 'lvm' plugins. Specific
+ * version of the lvm plugin is required:
+ *
+ * |[<!-- language="C" -->
+ * GError *error = NULL;
+ * gboolean ret = FALSE;
+ * BDPluginSpec fs_plugin = {BD_PLUGIN_FS, NULL};
+ * BDPluginSpec lvm_plugin = {BD_PLUGIN_LVM, "libbd_lvm.so.2"};
+ *
+ * BDPluginSpec *plugins[] = {&fs_plugin, &lvm_plugin, NULL};
+ *
+ * ret = bd_init (plugins, NULL, &error);
+ * ]|
  */
 gboolean bd_init (BDPluginSpec **require_plugins, BDUtilsLogFunc log_func, GError **error) {
     gboolean success = TRUE;
@@ -410,10 +443,10 @@ gboolean bd_init (BDPluginSpec **require_plugins, BDUtilsLogFunc log_func, GErro
  * @require_plugins plugins are available, tries to (re)initialize it. Otherwise
  * just returns early. The difference between:
  *
- * <code>
+ * |[<!-- language="C" -->
  * if (!bd_is_initialized())
  *     bd_init(None, None, &error);
- * </code>
+ * ]|
  *
  * and this function is that this function does the check and init in an atomic
  * way (holding the lock preventing other threads from doing changes in
@@ -703,3 +736,41 @@ gchar* bd_get_plugin_soname (BDPlugin plugin) {
 
     return NULL;
 }
+
+/**
+ * bd_get_plugin_name:
+ * @plugin: the queried plugin
+ *
+ * Returns: (transfer none): name of the plugin
+ */
+gchar* bd_get_plugin_name (BDPlugin plugin) {
+    return plugin_names[plugin];
+}
+
+/**
+ * bd_switch_init_checks:
+ * @enable: whether to enable init checks (%TRUE) or not (%FALSE)
+ * @error: (out): place to store error (if any)
+ *
+ * Enables or disables plugins' init checks based on @enable.
+ *
+ * Note: The current implementation (un)sets the LIBBLOCKDEV_SKIP_DEP_CHECKS
+ *       environment variable.
+ */
+gboolean bd_switch_init_checks (gboolean enable, GError **error) {
+    /* getenv/setenv/unsetenv are not thread-safe, better use a lock here */
+    g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&env_lock);
+
+    if (!enable && !g_getenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS")) {
+        if (!g_setenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS", "", FALSE)) {
+            g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_FAILED,
+                         "Failed to set the LIBBLOCKDEV_SKIP_DEP_CHECKS environment variable");
+            return FALSE;
+        }
+        return TRUE;
+    } else if (enable && g_getenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS")) {
+        g_unsetenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS");
+        return TRUE;
+    } else
+        return TRUE;
+}
