@@ -1,3 +1,21 @@
+GQuark  bd_md_error_quark (void) {
+        return g_quark_from_static_string ("g-bd-md-error-quark");
+}
+
+/**
+ * BDMDExamineData:
+ * @device: path of the MD device
+ * @level: RAID level of the device
+ * @num_devices: number of devices used by the MD device
+ * @name: name of the MD device
+ * @size: size of the MD device
+ * @uuid: array UUID
+ * @update_time: update time of the MD device
+ * @dev_uuid: UUID of the member device
+ * @events: number of events on the MD device
+ * @metadata: version of the metadata used by the MD device
+ * @chunk_size: chunk size used by the MD device
+ */
 /**
  * bd_md_examine_data_copy: (skip)
  *
@@ -16,6 +34,7 @@ BDMDExamineData* bd_md_examine_data_copy (BDMDExamineData *data) {
     new_data->dev_uuid = g_strdup (data->dev_uuid);
     new_data->events = data->events;
     new_data->metadata = g_strdup (data->metadata);
+    new_data->chunk_size = data->chunk_size;
     return new_data;
 }
 
@@ -46,6 +65,24 @@ GType  bd_md_examine_data_get_type () {
     return type;
 }
 
+/**
+ * BDMDDetailData:
+ * @device: path of the device
+ * @metadata: version of the metadata used by the device
+ * @creation_time: creation time
+ * @level: level of the MD RAID
+ * @name: name of the MD device
+ * @array_size: size of the MD array
+ * @use_dev_size: size of the used space
+ * @raid_devices: number of devices in the MD array
+ * @total_devices: total number of devices in the MD array
+ * @active_devices: number of active devices in the MD array
+ * @working_devices: number of working devices in the MD array
+ * @failed_devices: number of failed devices in the MD array
+ * @spare_devices: number of spare devices in the MD array
+ * @clean: whether the MD array is clean or not
+ * @uuid: uuid of the MD array
+ */
 /**
  * bd_md_detail_data_copy: (skip)
  *
@@ -101,6 +138,7 @@ GType  bd_md_detail_data_get_type () {
 }
 
 static const gchar const * const mdraid_functions[] = {
+    "bd_md_is_tech_avail",
     "bd_md_get_superblock_size",
     "bd_md_create",
     "bd_md_destroy",
@@ -117,26 +155,53 @@ static const gchar const * const mdraid_functions[] = {
     "bd_md_get_md_uuid",
     "bd_md_node_from_name",
     "bd_md_name_from_node",
+    "bd_md_get_status",
+    "bd_md_set_bitmap_location",
+    "bd_md_get_bitmap_location",
+    "bd_md_request_sync_action",
     NULL};
 
 gchar const * const * get_mdraid_functions (void) {
     return mdraid_functions;
 }
 
-static const guint8 mdraid_num_functions = 16;
+static const guint8 mdraid_num_functions = 21;
 
 guint8 get_mdraid_num_functions (void) {
     return mdraid_num_functions;
 }
 
-guint64  bd_md_get_superblock_size_stub (guint64 member_size, gchar *version, GError **error) {
+gboolean  bd_md_is_tech_avail_stub (BDMDTech tech, guint64 mode, GError **error) {
+    g_critical ("The function 'bd_md_is_tech_avail' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_md_is_tech_avail' called, but not implemented!");
+    return FALSE;
+}
+
+gboolean  (*_bd_md_is_tech_avail) (BDMDTech tech, guint64 mode, GError **error) = bd_md_is_tech_avail_stub;
+
+/**
+ * bd_md_is_tech_avail:
+ * @tech: the queried tech
+ * @mode: a bit mask of queried modes of operation for @tech
+ * @error: (out): place to store error (details about why the @tech-@mode combination is not available)
+ *
+ * Returns: whether the @tech-@mode combination is available -- supported by the
+ *          plugin implementation and having all the runtime dependencies available
+ */
+gboolean  bd_md_is_tech_avail (BDMDTech tech, guint64 mode, GError **error) {
+    return _bd_md_is_tech_avail (tech, mode, error);
+}
+
+
+guint64  bd_md_get_superblock_size_stub (guint64 member_size, const gchar *version, GError **error) {
     g_critical ("The function 'bd_md_get_superblock_size' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_get_superblock_size' called, but not implemented!");
     return 0;
 }
 
-guint64  (*_bd_md_get_superblock_size) (guint64 member_size, gchar *version, GError **error) = bd_md_get_superblock_size_stub;
+guint64  (*_bd_md_get_superblock_size) (guint64 member_size, const gchar *version, GError **error) = bd_md_get_superblock_size_stub;
 
 /**
  * bd_md_get_superblock_size:
@@ -146,20 +211,22 @@ guint64  (*_bd_md_get_superblock_size) (guint64 member_size, gchar *version, GEr
  *
  * Returns: Calculated superblock size for an array with a given @member_size
  * and metadata @version or default if unsupported @version is used.
+ *
+ * Tech category: always available
  */
-guint64  bd_md_get_superblock_size (guint64 member_size, gchar *version, GError **error) {
+guint64  bd_md_get_superblock_size (guint64 member_size, const gchar *version, GError **error) {
     return _bd_md_get_superblock_size (member_size, version, error);
 }
 
 
-gboolean  bd_md_create_stub (gchar *device_name, gchar *level, gchar **disks, guint64 spares, gchar *version, gboolean bitmap, GError **error) {
+gboolean  bd_md_create_stub (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, gboolean bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) {
     g_critical ("The function 'bd_md_create' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_create' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_create) (gchar *device_name, gchar *level, gchar **disks, guint64 spares, gchar *version, gboolean bitmap, GError **error) = bd_md_create_stub;
+gboolean  (*_bd_md_create) (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, gboolean bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) = bd_md_create_stub;
 
 /**
  * bd_md_create:
@@ -169,23 +236,28 @@ gboolean  (*_bd_md_create) (gchar *device_name, gchar *level, gchar **disks, gui
  * @spares: number of spare devices
  * @version: (allow-none): metadata version
  * @bitmap: whether to create an internal bitmap on the device or not
+ * @chunk_size: chunk size of the device to create
+ * @extra: (allow-none) (array zero-terminated=1): extra options for the creation (right now
+ *                                                 passed to the 'mdadm' utility)
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the new MD RAID device @device_name was successfully created or not
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_CREATE
  */
-gboolean  bd_md_create (gchar *device_name, gchar *level, gchar **disks, guint64 spares, gchar *version, gboolean bitmap, GError **error) {
-    return _bd_md_create (device_name, level, disks, spares, version, bitmap, error);
+gboolean  bd_md_create (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, gboolean bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) {
+    return _bd_md_create (device_name, level, disks, spares, version, bitmap, chunk_size, extra, error);
 }
 
 
-gboolean  bd_md_destroy_stub (gchar *device, GError **error) {
+gboolean  bd_md_destroy_stub (const gchar *device, GError **error) {
     g_critical ("The function 'bd_md_destroy' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_destroy' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_destroy) (gchar *device, GError **error) = bd_md_destroy_stub;
+gboolean  (*_bd_md_destroy) (const gchar *device, GError **error) = bd_md_destroy_stub;
 
 /**
  * bd_md_destroy:
@@ -193,87 +265,98 @@ gboolean  (*_bd_md_destroy) (gchar *device, GError **error) = bd_md_destroy_stub
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the MD RAID metadata was successfully destroyed on @device or not
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_DELETE
  */
-gboolean  bd_md_destroy (gchar *device, GError **error) {
+gboolean  bd_md_destroy (const gchar *device, GError **error) {
     return _bd_md_destroy (device, error);
 }
 
 
-gboolean  bd_md_deactivate_stub (gchar *device_name, GError **error) {
+gboolean  bd_md_deactivate_stub (const gchar *raid_spec, GError **error) {
     g_critical ("The function 'bd_md_deactivate' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_deactivate' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_deactivate) (gchar *device_name, GError **error) = bd_md_deactivate_stub;
+gboolean  (*_bd_md_deactivate) (const gchar *raid_spec, GError **error) = bd_md_deactivate_stub;
 
 /**
  * bd_md_deactivate:
- * @device_name: name of the RAID device to deactivate
+ * @raid_spec: specification of the RAID device (name, node or path)
  * @error: (out): place to store error (if any)
  *
- * Returns: whether the RAID device @device_name was successfully deactivated or not
+ * Returns: whether the RAID device @raid_spec was successfully deactivated or not
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
  */
-gboolean  bd_md_deactivate (gchar *device_name, GError **error) {
-    return _bd_md_deactivate (device_name, error);
+gboolean  bd_md_deactivate (const gchar *raid_spec, GError **error) {
+    return _bd_md_deactivate (raid_spec, error);
 }
 
 
-gboolean  bd_md_activate_stub (gchar *device_name, gchar **members, gchar *uuid, GError **error) {
+gboolean  bd_md_activate_stub (const gchar *raid_spec, const gchar **members, const gchar *uuid, gboolean start_degraded, const BDExtraArg **extra, GError **error) {
     g_critical ("The function 'bd_md_activate' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_activate' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_activate) (gchar *device_name, gchar **members, gchar *uuid, GError **error) = bd_md_activate_stub;
+gboolean  (*_bd_md_activate) (const gchar *raid_spec, const gchar **members, const gchar *uuid, gboolean start_degraded, const BDExtraArg **extra, GError **error) = bd_md_activate_stub;
 
 /**
  * bd_md_activate:
- * @device_name: name of the RAID device to activate
+ * @raid_spec: (allow-none): specification of the RAID device (name, node or path) to activate (if not given "--scan" is implied and @members is ignored)
  * @members: (allow-none) (array zero-terminated=1): member devices to be considered for @device activation
  * @uuid: (allow-none): UUID (in the MD RAID format!) of the MD RAID to activate
+ * @start_degraded: whether to start the array even if it's degraded
+ * @extra: (allow-none) (array zero-terminated=1): extra options for the activation (right now
+ *                                                 passed to the 'mdadm' utility)
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the MD RAID @device was successfully activated or not
  *
  * Note: either @members or @uuid (or both) have to be specified.
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
  */
-gboolean  bd_md_activate (gchar *device_name, gchar **members, gchar *uuid, GError **error) {
-    return _bd_md_activate (device_name, members, uuid, error);
+gboolean  bd_md_activate (const gchar *raid_spec, const gchar **members, const gchar *uuid, gboolean start_degraded, const BDExtraArg **extra, GError **error) {
+    return _bd_md_activate (raid_spec, members, uuid, start_degraded, extra, error);
 }
 
 
-gboolean  bd_md_run_stub (gchar *raid_name, GError **error) {
+gboolean  bd_md_run_stub (const gchar *raid_spec, GError **error) {
     g_critical ("The function 'bd_md_run' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_run' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_run) (gchar *raid_name, GError **error) = bd_md_run_stub;
+gboolean  (*_bd_md_run) (const gchar *raid_spec, GError **error) = bd_md_run_stub;
 
 /**
  * bd_md_run:
- * @raid_name: name of the (possibly degraded) MD RAID to be started
+ * @raid_spec: specification of the (possibly degraded) RAID device (name, node or path) to be started
  * @error: (out): place to store error (if any)
  *
- * Returns: whether the @raid_name was successfully started or not
+ * Returns: whether the @raid_spec was successfully started or not
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
  */
-gboolean  bd_md_run (gchar *raid_name, GError **error) {
-    return _bd_md_run (raid_name, error);
+gboolean  bd_md_run (const gchar *raid_spec, GError **error) {
+    return _bd_md_run (raid_spec, error);
 }
 
 
-gboolean  bd_md_nominate_stub (gchar *device, GError **error) {
+gboolean  bd_md_nominate_stub (const gchar *device, GError **error) {
     g_critical ("The function 'bd_md_nominate' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_nominate' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_nominate) (gchar *device, GError **error) = bd_md_nominate_stub;
+gboolean  (*_bd_md_nominate) (const gchar *device, GError **error) = bd_md_nominate_stub;
 
 /**
  * bd_md_nominate:
@@ -284,20 +367,22 @@ gboolean  (*_bd_md_nominate) (gchar *device, GError **error) = bd_md_nominate_st
  * appropriate RAID) or not
  *
  * Note: may start the MD RAID if it becomes ready by adding @device.
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
  */
-gboolean  bd_md_nominate (gchar *device, GError **error) {
+gboolean  bd_md_nominate (const gchar *device, GError **error) {
     return _bd_md_nominate (device, error);
 }
 
 
-gboolean  bd_md_denominate_stub (gchar *device, GError **error) {
+gboolean  bd_md_denominate_stub (const gchar *device, GError **error) {
     g_critical ("The function 'bd_md_denominate' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_denominate' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_denominate) (gchar *device, GError **error) = bd_md_denominate_stub;
+gboolean  (*_bd_md_denominate) (const gchar *device, GError **error) = bd_md_denominate_stub;
 
 /**
  * bd_md_denominate:
@@ -308,30 +393,34 @@ gboolean  (*_bd_md_denominate) (gchar *device, GError **error) = bd_md_denominat
  * appropriate RAID) or not
  *
  * Note: may start the MD RAID if it becomes ready by adding @device.
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
  */
-gboolean  bd_md_denominate (gchar *device, GError **error) {
+gboolean  bd_md_denominate (const gchar *device, GError **error) {
     return _bd_md_denominate (device, error);
 }
 
 
-gboolean  bd_md_add_stub (gchar *raid_name, gchar *device, guint64 raid_devs, GError **error) {
+gboolean  bd_md_add_stub (const gchar *raid_spec, const gchar *device, guint64 raid_devs, const BDExtraArg **extra, GError **error) {
     g_critical ("The function 'bd_md_add' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_add' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_add) (gchar *raid_name, gchar *device, guint64 raid_devs, GError **error) = bd_md_add_stub;
+gboolean  (*_bd_md_add) (const gchar *raid_spec, const gchar *device, guint64 raid_devs, const BDExtraArg **extra, GError **error) = bd_md_add_stub;
 
 /**
  * bd_md_add:
- * @raid_name: name of the RAID device to add @device into
- * @device: name of the device to add to the @raid_name RAID device
- * @raid_devs: number of devices the @raid_name RAID should actively use (see
- *             below) or 0 to leave unspecified
+ * @raid_spec: specification of the RAID device (name, node or path) to add @device into
+ * @device: name of the device to add to the @raid_spec RAID device
+ * @raid_devs: number of devices the @raid_spec RAID should actively use or 0
+ *             to leave unspecified (see below)
+ * @extra: (allow-none) (array zero-terminated=1): extra options for the addition (right now
+ *                                                 passed to the 'mdadm' utility)
  * @error: (out): place to store error (if any)
  *
- * Returns: whether the @device was successfully added to the @raid_name RAID or
+ * Returns: whether the @device was successfully added to the @raid_spec RAID or
  * not
  *
  * The @raid_devs parameter is used when adding devices to a raid array that has
@@ -340,44 +429,50 @@ gboolean  (*_bd_md_add) (gchar *raid_name, gchar *device, guint64 raid_devs, GEr
  *
  * Whether the new device will be added as a spare or an active member is
  * decided by mdadm.
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
  */
-gboolean  bd_md_add (gchar *raid_name, gchar *device, guint64 raid_devs, GError **error) {
-    return _bd_md_add (raid_name, device, raid_devs, error);
+gboolean  bd_md_add (const gchar *raid_spec, const gchar *device, guint64 raid_devs, const BDExtraArg **extra, GError **error) {
+    return _bd_md_add (raid_spec, device, raid_devs, extra, error);
 }
 
 
-gboolean  bd_md_remove_stub (gchar *raid_name, gchar *device, gboolean fail, GError **error) {
+gboolean  bd_md_remove_stub (const gchar *raid_spec, const gchar *device, gboolean fail, const BDExtraArg **extra, GError **error) {
     g_critical ("The function 'bd_md_remove' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_remove' called, but not implemented!");
     return FALSE;
 }
 
-gboolean  (*_bd_md_remove) (gchar *raid_name, gchar *device, gboolean fail, GError **error) = bd_md_remove_stub;
+gboolean  (*_bd_md_remove) (const gchar *raid_spec, const gchar *device, gboolean fail, const BDExtraArg **extra, GError **error) = bd_md_remove_stub;
 
 /**
  * bd_md_remove:
- * @raid_name: name of the RAID device to remove @device from
- * @device: device to remove from the @raid_name RAID
+ * @raid_spec: specification of the RAID device (name, node or path) to remove @device from
+ * @device: device to remove from the @raid_spec RAID
  * @fail: whether to mark the @device as failed before removing
+ * @extra: (allow-none) (array zero-terminated=1): extra options for the removal (right now
+ *                                                 passed to the 'mdadm' utility)
  * @error: (out): place to store error (if any)
  *
- * Returns: whether the @device was successfully removed from the @raid_name
+ * Returns: whether the @device was successfully removed from the @raid_spec
  * RAID or not.
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
  */
-gboolean  bd_md_remove (gchar *raid_name, gchar *device, gboolean fail, GError **error) {
-    return _bd_md_remove (raid_name, device, fail, error);
+gboolean  bd_md_remove (const gchar *raid_spec, const gchar *device, gboolean fail, const BDExtraArg **extra, GError **error) {
+    return _bd_md_remove (raid_spec, device, fail, extra, error);
 }
 
 
-BDMDExamineData* bd_md_examine_stub (gchar *device, GError **error) {
+BDMDExamineData* bd_md_examine_stub (const gchar *device, GError **error) {
     g_critical ("The function 'bd_md_examine' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_examine' called, but not implemented!");
     return NULL;
 }
 
-BDMDExamineData* (*_bd_md_examine) (gchar *device, GError **error) = bd_md_examine_stub;
+BDMDExamineData* (*_bd_md_examine) (const gchar *device, GError **error) = bd_md_examine_stub;
 
 /**
  * bd_md_examine:
@@ -385,41 +480,45 @@ BDMDExamineData* (*_bd_md_examine) (gchar *device, GError **error) = bd_md_exami
  * @error: (out): place to store error (if any)
  *
  * Returns: information about the MD RAID extracted from the @device
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_QUERY
  */
-BDMDExamineData* bd_md_examine (gchar *device, GError **error) {
+BDMDExamineData* bd_md_examine (const gchar *device, GError **error) {
     return _bd_md_examine (device, error);
 }
 
 
-BDMDDetailData* bd_md_detail_stub (gchar *raid_name, GError **error) {
+BDMDDetailData* bd_md_detail_stub (const gchar *raid_spec, GError **error) {
     g_critical ("The function 'bd_md_detail' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_detail' called, but not implemented!");
     return NULL;
 }
 
-BDMDDetailData* (*_bd_md_detail) (gchar *raid_name, GError **error) = bd_md_detail_stub;
+BDMDDetailData* (*_bd_md_detail) (const gchar *raid_spec, GError **error) = bd_md_detail_stub;
 
 /**
  * bd_md_detail:
- * @raid_name: name of the MD RAID to examine
+ * @raid_spec: specification of the RAID device (name, node or path) to examine
  * @error: (out): place to store error (if any)
  *
- * Returns: information about the MD RAID @raid_name
+ * Returns: information about the MD RAID @raid_spec
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_QUERY
  */
-BDMDDetailData* bd_md_detail (gchar *raid_name, GError **error) {
-    return _bd_md_detail (raid_name, error);
+BDMDDetailData* bd_md_detail (const gchar *raid_spec, GError **error) {
+    return _bd_md_detail (raid_spec, error);
 }
 
 
-gchar* bd_md_canonicalize_uuid_stub (gchar *uuid, GError **error) {
+gchar* bd_md_canonicalize_uuid_stub (const gchar *uuid, GError **error) {
     g_critical ("The function 'bd_md_canonicalize_uuid' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_canonicalize_uuid' called, but not implemented!");
     return NULL;
 }
 
-gchar* (*_bd_md_canonicalize_uuid) (gchar *uuid, GError **error) = bd_md_canonicalize_uuid_stub;
+gchar* (*_bd_md_canonicalize_uuid) (const gchar *uuid, GError **error) = bd_md_canonicalize_uuid_stub;
 
 /**
  * bd_md_canonicalize_uuid:
@@ -430,20 +529,22 @@ gchar* (*_bd_md_canonicalize_uuid) (gchar *uuid, GError **error) = bd_md_canonic
  *
  * This function expects a UUID in the form that mdadm returns. The change is as
  * follows: 3386ff85:f5012621:4a435f06:1eb47236 -> 3386ff85-f501-2621-4a43-5f061eb47236
+ *
+ * Tech category: always available
  */
-gchar* bd_md_canonicalize_uuid (gchar *uuid, GError **error) {
+gchar* bd_md_canonicalize_uuid (const gchar *uuid, GError **error) {
     return _bd_md_canonicalize_uuid (uuid, error);
 }
 
 
-gchar* bd_md_get_md_uuid_stub (gchar *uuid, GError **error) {
+gchar* bd_md_get_md_uuid_stub (const gchar *uuid, GError **error) {
     g_critical ("The function 'bd_md_get_md_uuid' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_get_md_uuid' called, but not implemented!");
     return NULL;
 }
 
-gchar* (*_bd_md_get_md_uuid) (gchar *uuid, GError **error) = bd_md_get_md_uuid_stub;
+gchar* (*_bd_md_get_md_uuid) (const gchar *uuid, GError **error) = bd_md_get_md_uuid_stub;
 
 /**
  * bd_md_get_md_uuid:
@@ -456,41 +557,45 @@ gchar* (*_bd_md_get_md_uuid) (gchar *uuid, GError **error) = bd_md_get_md_uuid_s
  * returns a UUID in the format used by MD RAID and is thus reverse to
  * bd_md_canonicalize_uuid(). The change is as follows:
  * 3386ff85-f501-2621-4a43-5f061eb47236 -> 3386ff85:f5012621:4a435f06:1eb47236
+ *
+ * Tech category: always available
  */
-gchar* bd_md_get_md_uuid (gchar *uuid, GError **error) {
+gchar* bd_md_get_md_uuid (const gchar *uuid, GError **error) {
     return _bd_md_get_md_uuid (uuid, error);
 }
 
 
-gchar* bd_md_node_from_name_stub (gchar *name, GError **error) {
+gchar* bd_md_node_from_name_stub (const gchar *name, GError **error) {
     g_critical ("The function 'bd_md_node_from_name' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_node_from_name' called, but not implemented!");
     return NULL;
 }
 
-gchar* (*_bd_md_node_from_name) (gchar *name, GError **error) = bd_md_node_from_name_stub;
+gchar* (*_bd_md_node_from_name) (const gchar *name, GError **error) = bd_md_node_from_name_stub;
 
 /**
  * bd_md_node_from_name:
  * @name: name of the MD RAID
  * @error: (out): place to store error (if any)
  *
- * Returns: path to the @name MD RAID's device node or %NULL in case of error
+ * Returns: device node of the @name MD RAID or %NULL in case of error
+ *
+ * Tech category: always available
  */
-gchar* bd_md_node_from_name (gchar *name, GError **error) {
+gchar* bd_md_node_from_name (const gchar *name, GError **error) {
     return _bd_md_node_from_name (name, error);
 }
 
 
-gchar* bd_md_name_from_node_stub (gchar *node, GError **error) {
+gchar* bd_md_name_from_node_stub (const gchar *node, GError **error) {
     g_critical ("The function 'bd_md_name_from_node' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_name_from_node' called, but not implemented!");
     return NULL;
 }
 
-gchar* (*_bd_md_name_from_node) (gchar *node, GError **error) = bd_md_name_from_node_stub;
+gchar* (*_bd_md_name_from_node) (const gchar *node, GError **error) = bd_md_name_from_node_stub;
 
 /**
  * bd_md_name_from_node:
@@ -498,13 +603,110 @@ gchar* (*_bd_md_name_from_node) (gchar *node, GError **error) = bd_md_name_from_
  * @error: (out): place to store error (if any)
  *
  * Returns: @name of the MD RAID the device node belongs to or %NULL in case of error
+ *
+ * Tech category: always available
  */
-gchar* bd_md_name_from_node (gchar *node, GError **error) {
+gchar* bd_md_name_from_node (const gchar *node, GError **error) {
     return _bd_md_name_from_node (node, error);
 }
 
 
-gpointer load_mdraid_from_plugin(gchar *so_name) {
+gchar* bd_md_get_status_stub (const gchar *raid_spec, GError **error) {
+    g_critical ("The function 'bd_md_get_status' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_md_get_status' called, but not implemented!");
+    return NULL;
+}
+
+gchar* (*_bd_md_get_status) (const gchar *raid_spec, GError **error) = bd_md_get_status_stub;
+
+/**
+ * bd_md_get_status
+ * @raid_spec: specification of the RAID device (name, node or path) to get status
+ * @error: (out): place to store error (if any)
+ *
+ * Returns: (transfer full): status of the @raid_spec RAID.
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_QUERY
+ */
+gchar* bd_md_get_status (const gchar *raid_spec, GError **error) {
+    return _bd_md_get_status (raid_spec, error);
+}
+
+
+gboolean  bd_md_set_bitmap_location_stub (const gchar *raid_spec, const gchar *location, GError **error) {
+    g_critical ("The function 'bd_md_set_bitmap_location' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_md_set_bitmap_location' called, but not implemented!");
+    return FALSE;
+}
+
+gboolean  (*_bd_md_set_bitmap_location) (const gchar *raid_spec, const gchar *location, GError **error) = bd_md_set_bitmap_location_stub;
+
+/**
+ * bd_md_set_bitmap_location:
+ * @raid_spec: specification of the RAID device (name, node or path) to set the bitmap location
+ * @location: bitmap location (none, internal or path)
+ * @error: (out): place to store error (if any)
+ *
+ * Returns: whether @location was successfully set for @raid_spec
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
+ */
+gboolean  bd_md_set_bitmap_location (const gchar *raid_spec, const gchar *location, GError **error) {
+    return _bd_md_set_bitmap_location (raid_spec, location, error);
+}
+
+
+gchar* bd_md_get_bitmap_location_stub (const gchar *raid_spec, GError **error) {
+    g_critical ("The function 'bd_md_get_bitmap_location' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_md_get_bitmap_location' called, but not implemented!");
+    return NULL;
+}
+
+gchar* (*_bd_md_get_bitmap_location) (const gchar *raid_spec, GError **error) = bd_md_get_bitmap_location_stub;
+
+/**
+ * bd_md_get_bitmap_location:
+ * @raid_spec: specification of the RAID device (name, node or path) to get the bitmap location
+ * @error: (out): place to store error (if any)
+ *
+ * Returns: (transfer full): bitmap location for @raid_spec
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_QUERY
+ */
+gchar* bd_md_get_bitmap_location (const gchar *raid_spec, GError **error) {
+    return _bd_md_get_bitmap_location (raid_spec, error);
+}
+
+
+gboolean  bd_md_request_sync_action_stub (const gchar *raid_spec, const gchar *action, GError **error) {
+    g_critical ("The function 'bd_md_request_sync_action' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_md_request_sync_action' called, but not implemented!");
+    return FALSE;
+}
+
+gboolean  (*_bd_md_request_sync_action) (const gchar *raid_spec, const gchar *action, GError **error) = bd_md_request_sync_action_stub;
+
+/**
+ * bd_md_request_sync_action:
+ * @raid_spec: specification of the RAID device (name, node or path) to request sync action on
+ * @action: requested sync action (resync, recovery, check, repair or idle)
+ * @error: (out): place to store error (if any)
+ *
+ * Returns: whether the @action was successfully requested for the @raid_spec
+ * RAID or not.
+ *
+ * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_MODIFY
+ */
+gboolean  bd_md_request_sync_action (const gchar *raid_spec, const gchar *action, GError **error) {
+    return _bd_md_request_sync_action (raid_spec, action, error);
+}
+
+
+gpointer load_mdraid_from_plugin(const gchar *so_name) {
     void *handle = NULL;
     char *error = NULL;
     gboolean (*check_fn) (void) = NULL;
@@ -517,16 +719,16 @@ gpointer load_mdraid_from_plugin(gchar *so_name) {
     }
 
     dlerror();
-    * (void**) (&check_fn) = dlsym(handle, "check");
+    * (void**) (&check_fn) = dlsym(handle, "bd_md_check_deps");
     if ((error = dlerror()) != NULL)
         g_debug("failed to load the check() function for mdraid: %s", error);
-    if (check_fn && !check_fn()) {
+    if (!g_getenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS") && check_fn && !check_fn()) {
         dlclose(handle);
         return NULL;
     }    check_fn = NULL;
 
     dlerror();
-    * (void**) (&init_fn) = dlsym(handle, "init");
+    * (void**) (&init_fn) = dlsym(handle, "bd_md_init");
     if ((error = dlerror()) != NULL)
         g_debug("failed to load the init() function for mdraid: %s", error);
     if (init_fn && !init_fn()) {
@@ -534,6 +736,11 @@ gpointer load_mdraid_from_plugin(gchar *so_name) {
         return NULL;
     }    init_fn = NULL;
 
+    dlerror();
+    * (void**) (&_bd_md_is_tech_avail) = dlsym(handle, "bd_md_is_tech_avail");
+    if ((error = dlerror()) != NULL)
+        g_warning("failed to load bd_md_is_tech_avail: %s", error);
+
     dlerror();
     * (void**) (&_bd_md_get_superblock_size) = dlsym(handle, "bd_md_get_superblock_size");
     if ((error = dlerror()) != NULL)
@@ -614,10 +821,34 @@ gpointer load_mdraid_from_plugin(gchar *so_name) {
     if ((error = dlerror()) != NULL)
         g_warning("failed to load bd_md_name_from_node: %s", error);
 
+    dlerror();
+    * (void**) (&_bd_md_get_status) = dlsym(handle, "bd_md_get_status");
+    if ((error = dlerror()) != NULL)
+        g_warning("failed to load bd_md_get_status: %s", error);
+
+    dlerror();
+    * (void**) (&_bd_md_set_bitmap_location) = dlsym(handle, "bd_md_set_bitmap_location");
+    if ((error = dlerror()) != NULL)
+        g_warning("failed to load bd_md_set_bitmap_location: %s", error);
+
+    dlerror();
+    * (void**) (&_bd_md_get_bitmap_location) = dlsym(handle, "bd_md_get_bitmap_location");
+    if ((error = dlerror()) != NULL)
+        g_warning("failed to load bd_md_get_bitmap_location: %s", error);
+
+    dlerror();
+    * (void**) (&_bd_md_request_sync_action) = dlsym(handle, "bd_md_request_sync_action");
+    if ((error = dlerror()) != NULL)
+        g_warning("failed to load bd_md_request_sync_action: %s", error);
+
     return handle;
 }
 
 gboolean unload_mdraid (gpointer handle) {
+    char *error = NULL;
+    gboolean (*close_fn) (void) = NULL;
+
+    _bd_md_is_tech_avail = bd_md_is_tech_avail_stub;
     _bd_md_get_superblock_size = bd_md_get_superblock_size_stub;
     _bd_md_create = bd_md_create_stub;
     _bd_md_destroy = bd_md_destroy_stub;
@@ -634,6 +865,18 @@ gboolean unload_mdraid (gpointer handle) {
     _bd_md_get_md_uuid = bd_md_get_md_uuid_stub;
     _bd_md_node_from_name = bd_md_node_from_name_stub;
     _bd_md_name_from_node = bd_md_name_from_node_stub;
+    _bd_md_get_status = bd_md_get_status_stub;
+    _bd_md_set_bitmap_location = bd_md_set_bitmap_location_stub;
+    _bd_md_get_bitmap_location = bd_md_get_bitmap_location_stub;
+    _bd_md_request_sync_action = bd_md_request_sync_action_stub;
+
+    dlerror();
+    * (void**) (&close_fn) = dlsym(handle, "bd_md_close");
+    if (((error = dlerror()) != NULL) || !close_fn)
+        g_debug("failed to load the close_plugin() function for mdraid: %s", error);
+    if (close_fn) {
+        close_fn();
+    }
 
     return dlclose(handle) == 0;
 }
