@@ -19,12 +19,13 @@
 
 #include <glib.h>
 #include <unistd.h>
-#include <utils.h>
+#include <blockdev/utils.h>
 #include <libdevmapper.h>
 #include <dmraid/dmraid.h>
 #include <libudev.h>
 
 #include "dm.h"
+#include "check_deps.h"
 
 /* macros taken from the pyblock/dmraid.h file plus one more*/
 #define for_each_raidset(_c, _n) list_for_each_entry(_n, LC_RS(_c), list)
@@ -50,6 +51,19 @@ GQuark bd_dm_error_quark (void)
 
 typedef struct raid_set* (*RSEvalFunc) (struct raid_set *rs, gpointer data);
 
+
+static volatile guint avail_deps = 0;
+static GMutex deps_check_lock;
+
+#define DEPS_DMSETUP 0
+#define DEPS_DMSETUP_MASK (1 << DEPS_DMSETUP)
+#define DEPS_LAST 1
+
+static UtilDep deps[DEPS_LAST] = {
+    {"dmsetup", DM_MIN_VERSION, NULL, "Library version:\\s+([\\d\\.]+)"},
+};
+
+
 /**
  * discard_dm_log: (skip)
  */
@@ -59,9 +73,44 @@ static void discard_dm_log (int level __attribute__((unused)), const char *file
 }
 
 /**
- * init: (skip)
+ * bd_dm_check_deps:
+ *
+ * Returns: whether the plugin's runtime dependencies are satisfied or not
+ *
+ * Function checking plugin's runtime dependencies.
+ *
  */
-gboolean init () {
+gboolean bd_dm_check_deps () {
+    GError *error = NULL;
+    guint i = 0;
+    gboolean status = FALSE;
+    gboolean ret = TRUE;
+
+    for (i=0; i < DEPS_LAST; i++) {
+        status = bd_utils_check_util_version (deps[i].name, deps[i].version,
+                                              deps[i].ver_arg, deps[i].ver_regexp, &error);
+        if (!status)
+            g_warning ("%s", error->message);
+        else
+            g_atomic_int_or (&avail_deps, 1 << i);
+        g_clear_error (&error);
+        ret = ret && status;
+    }
+
+    if (!ret)
+        g_warning("Cannot load the DM plugin");
+
+    return ret;
+}
+
+/**
+ * bd_dm_init:
+ *
+ * Initializes the plugin. **This function is called automatically by the
+ * library's initialization functions.**
+ *
+ */
+gboolean bd_dm_init () {
     dm_log_with_errno_init ((dm_log_with_errno_fn) discard_dm_log);
     dm_log_init_verbose (0);
 
@@ -69,17 +118,35 @@ gboolean init () {
 }
 
 /**
- * check: (skip)
+ * bd_dm_close:
+ *
+ * Cleans up after the plugin. **This function is called automatically by the
+ * library's functions that unload it.**
+ *
  */
-gboolean check() {
-    GError *error = NULL;
-    gboolean ret = bd_utils_check_util_version ("dmsetup", DM_MIN_VERSION, NULL, "Library version:\\s+([\\d\\.]+)", &error);
+void bd_dm_close () {
+    dm_log_with_errno_init (NULL);
+    dm_log_init_verbose (0);
+}
 
-    if (!ret && error) {
-        g_warning("Cannot load the DM plugin: %s" , error->message);
-        g_clear_error (&error);
-    }
-    return ret;
+#define UNUSED __attribute__((unused))
+
+/**
+ * bd_dm_is_tech_avail:
+ * @tech: the queried tech
+ * @mode: a bit mask of queried modes of operation (#BDDMTechMode) for @tech
+ * @error: (out): place to store error (details about why the @tech-@mode combination is not available)
+ *
+ * Returns: whether the @tech-@mode combination is avaible -- supported by the
+ *          plugin implementation and having all the runtime dependencies available
+ */
+gboolean bd_dm_is_tech_avail (BDDMTech tech, guint64 mode UNUSED, GError **error) {
+    /* all combinations are supported by this implementation of the plugin, but
+       BD_DM_TECH_MAP requires the 'dmsetup' utility */
+    if (tech == BD_DM_TECH_MAP)
+        return check_deps (&avail_deps, DEPS_DMSETUP_MASK, deps, DEPS_LAST, &deps_check_lock, error);
+    else
+        return TRUE;
 }
 
 /**
@@ -92,10 +159,15 @@ gboolean check() {
  *
  * Returns: whether the new linear mapping @map_name was successfully created
  * for the @device or not
+ *
+ * Tech category: %BD_DM_TECH_MAP-%BD_DM_TECH_MODE_CREATE_ACTIVATE
  */
-gboolean bd_dm_create_linear (gchar *map_name, gchar *device, guint64 length, gchar *uuid, GError **error) {
+gboolean bd_dm_create_linear (const gchar *map_name, const gchar *device, guint64 length, const gchar *uuid, GError **error) {
     gboolean success = FALSE;
-    gchar *argv[9] = {"dmsetup", "create", map_name, "--table", NULL, NULL, NULL, NULL, NULL};
+    const gchar *argv[9] = {"dmsetup", "create", map_name, "--table", NULL, NULL, NULL, NULL, NULL};
+
+    if (!check_deps (&avail_deps, DEPS_DMSETUP_MASK, deps, DEPS_LAST, &deps_check_lock, error))
+        return FALSE;
 
     gchar *table = g_strdup_printf ("0 %"G_GUINT64_FORMAT" linear %s 0", length, device);
     argv[4] = table;
@@ -107,7 +179,7 @@ gboolean bd_dm_create_linear (gchar *map_name, gchar *device, guint64 length, gc
     } else
         argv[5] = device;
 
-    success = bd_utils_exec_and_report_error (argv, error);
+    success = bd_utils_exec_and_report_error (argv, NULL, error);
     g_free (table);
 
     return success;
@@ -119,11 +191,16 @@ gboolean bd_dm_create_linear (gchar *map_name, gchar *device, guint64 length, gc
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the @map_name map was successfully removed or not
+ *
+ * Tech category: %BD_DM_TECH_MAP-%BD_DM_TECH_MODE_REMOVE_DEACTIVATE
  */
-gboolean bd_dm_remove (gchar *map_name, GError **error) {
-    gchar *argv[4] = {"dmsetup", "remove", map_name, NULL};
+gboolean bd_dm_remove (const gchar *map_name, GError **error) {
+    const gchar *argv[4] = {"dmsetup", "remove", map_name, NULL};
 
-    return bd_utils_exec_and_report_error (argv, error);
+    if (!check_deps (&avail_deps, DEPS_DMSETUP_MASK, deps, DEPS_LAST, &deps_check_lock, error))
+        return FALSE;
+
+    return bd_utils_exec_and_report_error (argv, NULL, error);
 }
 
 /**
@@ -133,8 +210,10 @@ gboolean bd_dm_remove (gchar *map_name, GError **error) {
  *
  * Returns: map name of the map providing the @dm_node device or %NULL
  * (@error) contains the error in such cases)
+ *
+ * Tech category: %BD_DM_TECH_MAP-%BD_DM_TECH_MODE_QUERY
  */
-gchar* bd_dm_name_from_node (gchar *dm_node, GError **error) {
+gchar* bd_dm_name_from_node (const gchar *dm_node, GError **error) {
     gchar *ret = NULL;
     gboolean success = FALSE;
 
@@ -165,24 +244,22 @@ gchar* bd_dm_name_from_node (gchar *dm_node, GError **error) {
  *
  * Returns: DM node name for the @map_name map or %NULL (@error) contains
  * the error in such cases)
+ *
+ * Tech category: %BD_DM_TECH_MAP-%BD_DM_TECH_MODE_QUERY
  */
-gchar* bd_dm_node_from_name (gchar *map_name, GError **error) {
-    gchar *symlink = NULL;
+gchar* bd_dm_node_from_name (const gchar *map_name, GError **error) {
+    gchar *dev_path = NULL;
     gchar *ret = NULL;
     gchar *dev_mapper_path = g_strdup_printf ("/dev/mapper/%s", map_name);
 
-    symlink = g_file_read_link (dev_mapper_path, error);
-    if (!symlink) {
-        /* error is already populated */
-        g_free (dev_mapper_path);
-        return NULL;
-    }
-
-    g_strstrip (symlink);
-    ret = g_path_get_basename (symlink);
-
-    g_free (symlink);
+    dev_path = bd_utils_resolve_device (dev_mapper_path, error);
     g_free (dev_mapper_path);
+    if (!dev_path)
+        /* error is already populated */
+        return NULL;
+
+    ret = g_path_get_basename (dev_path);
+    g_free (dev_path);
 
     return ret;
 }
@@ -197,8 +274,10 @@ gchar* bd_dm_node_from_name (gchar *map_name, GError **error) {
  * Returns: whether the given @map_name exists (and is live if @live_only is
  * %TRUE (and is active if @active_only is %TRUE)). If %FALSE is returned,
  * @error) indicates whether error appeared (non-%NULL) or not (%NULL).
+ *
+ * Tech category: %BD_DM_TECH_MAP-%BD_DM_TECH_MODE_QUERY
  */
-gboolean bd_dm_map_exists (gchar *map_name, gboolean live_only, gboolean active_only, GError **error) {
+gboolean bd_dm_map_exists (const gchar *map_name, gboolean live_only, gboolean active_only, GError **error) {
     struct dm_task *task_list = NULL;
     struct dm_task *task_info = NULL;
 	struct dm_names *names = NULL;
@@ -320,7 +399,7 @@ static struct lib_context* init_dmraid_stack (GError **error) {
  * Returns: whether the device specified by @sysname matches the spec given by @name,
  *          @uuid, @major and @minor
  */
-static gboolean raid_dev_matches_spec (struct raid_dev *raid_dev, gchar *name, gchar *uuid, gint major, gint minor) {
+static gboolean raid_dev_matches_spec (struct raid_dev *raid_dev, const gchar *name, const gchar *uuid, gint major, gint minor) {
     gchar const *dev_name = NULL;
     gchar const *dev_uuid;
     gchar const *major_str;
@@ -372,7 +451,7 @@ static gboolean raid_dev_matches_spec (struct raid_dev *raid_dev, gchar *name, g
 /**
  * find_raid_sets_for_dev: (skip)
  */
-static void find_raid_sets_for_dev (gchar *name, gchar *uuid, gint major, gint minor, struct lib_context *lc, struct raid_set *rs, GPtrArray *ret_sets) {
+static void find_raid_sets_for_dev (const gchar *name, const gchar *uuid, gint major, gint minor, struct lib_context *lc, struct raid_set *rs, GPtrArray *ret_sets) {
     struct raid_set *subset;
     struct raid_dev *dev;
 
@@ -399,8 +478,10 @@ static void find_raid_sets_for_dev (gchar *name, gchar *uuid, gint major, gint m
  * the member or %NULL in case of error
  *
  * One of @name, @uuid or @major:@minor has to be given.
+ *
+ * Tech category: %BD_DM_TECH_RAID-%BD_DM_TECH_MODE_QUERY
  */
-gchar** bd_dm_get_member_raid_sets (gchar *name, gchar *uuid, gint major, gint minor, GError **error) {
+gchar** bd_dm_get_member_raid_sets (const gchar *name, const gchar *uuid, gint major, gint minor, GError **error) {
     guint64 i = 0;
     struct lib_context *lc = NULL;
     struct raid_set *rs = NULL;
@@ -463,7 +544,7 @@ static struct raid_set* rs_matches_name (struct raid_set *rs, gpointer *name_dat
         return NULL;
 }
 
-static gboolean change_set_by_name (gchar *name, enum activate_type action, GError **error) {
+static gboolean change_set_by_name (const gchar *name, enum activate_type action, GError **error) {
     gint rc = 0;
     struct lib_context *lc;
     struct raid_set *iter_rs;
@@ -475,7 +556,7 @@ static gboolean change_set_by_name (gchar *name, enum activate_type action, GErr
         return FALSE;
 
     for_each_raidset (lc, iter_rs) {
-        match_rs = find_in_raid_sets (iter_rs, (RSEvalFunc)rs_matches_name, name);
+        match_rs = find_in_raid_sets (iter_rs, (RSEvalFunc)rs_matches_name, (gchar *)name);
         if (match_rs)
             break;
     }
@@ -505,9 +586,20 @@ static gboolean change_set_by_name (gchar *name, enum activate_type action, GErr
  * @error: (out): variable to store error (if any)
  *
  * Returns: whether the RAID set @name was successfully activate or not
+ *
+ * Tech category: %BD_DM_TECH_RAID-%BD_DM_TECH_CREATE_ACTIVATE
  */
-gboolean bd_dm_activate_raid_set (gchar *name, GError **error) {
-    return change_set_by_name (name, A_ACTIVATE, error);
+gboolean bd_dm_activate_raid_set (const gchar *name, GError **error) {
+    guint64 progress_id = 0;
+    gchar *msg = NULL;
+    gboolean ret = FALSE;
+
+    msg = g_strdup_printf ("Activating DM RAID set '%s'", name);
+    progress_id = bd_utils_report_started (msg);
+    g_free (msg);
+    ret = change_set_by_name (name, A_ACTIVATE, error);
+    bd_utils_report_finished (progress_id, "Completed");
+    return ret;
 }
 
 /**
@@ -516,9 +608,20 @@ gboolean bd_dm_activate_raid_set (gchar *name, GError **error) {
  * @error: (out): variable to store error (if any)
  *
  * Returns: whether the RAID set @name was successfully deactivate or not
+ *
+ * Tech category: %BD_DM_TECH_RAID-%BD_DM_TECH_REMOVE_DEACTIVATE
  */
-gboolean bd_dm_deactivate_raid_set (gchar *name, GError **error) {
-    return change_set_by_name (name, A_DEACTIVATE, error);
+gboolean bd_dm_deactivate_raid_set (const gchar *name, GError **error) {
+    guint64 progress_id = 0;
+    gchar *msg = NULL;
+    gboolean ret = FALSE;
+
+    msg = g_strdup_printf ("Deactivating DM RAID set '%s'", name);
+    progress_id = bd_utils_report_started (msg);
+    g_free (msg);
+    ret = change_set_by_name (name, A_DEACTIVATE, error);
+    bd_utils_report_finished (progress_id, "Completed");
+    return ret;
 }
 
 /**
@@ -527,8 +630,10 @@ gboolean bd_dm_deactivate_raid_set (gchar *name, GError **error) {
  * @error: (out): variable to store error (if any)
  *
  * Returns: string representation of the @name RAID set's type
+ *
+ * Tech category: %BD_DM_TECH_RAID-%BD_DM_TECH_QUERY
  */
-gchar* bd_dm_get_raid_set_type (gchar *name, GError **error) {
+gchar* bd_dm_get_raid_set_type (const gchar *name, GError **error) {
     struct lib_context *lc;
     struct raid_set *iter_rs;
     struct raid_set *match_rs = NULL;
@@ -540,7 +645,7 @@ gchar* bd_dm_get_raid_set_type (gchar *name, GError **error) {
         return NULL;
 
     for_each_raidset (lc, iter_rs) {
-        match_rs = find_in_raid_sets (iter_rs, (RSEvalFunc)rs_matches_name, name);
+        match_rs = find_in_raid_sets (iter_rs, (RSEvalFunc)rs_matches_name, (gchar *)name);
         if (match_rs)
             break;
     }
