@@ -27,6 +27,7 @@
 #include <stdio.h>
 
 #include "kbd.h"
+#include "check_deps.h"
 
 #define SECTOR_SIZE 512
 
@@ -39,15 +40,26 @@
  * A plugin for operations with kernel block devices.
  */
 
-#ifdef WITH_BD_BCACHE
 static const gchar * const mode_str[BD_KBD_MODE_UNKNOWN+1] = {"writethrough", "writeback", "writearound", "none", "unknown"};
-#endif
 
 /* "C" locale to get the locale-agnostic error messages */
 static locale_t c_locale = (locale_t) 0;
 
 static gboolean have_kernel_module (const gchar *module_name, GError **error);
 
+
+static volatile guint avail_deps = 0;
+static GMutex deps_check_lock;
+
+#define DEPS_MAKEBCACHE 0
+#define DEPS_MAKEBCACHE_MASK (1 << DEPS_MAKEBCACHE)
+#define DEPS_LAST 1
+
+static UtilDep deps[DEPS_LAST] = {
+    {"make-bcache", NULL, NULL, NULL},
+};
+
+
 /**
  * bd_kbd_check_deps:
  *
@@ -59,6 +71,8 @@ static gboolean have_kernel_module (const gchar *module_name, GError **error);
 gboolean bd_kbd_check_deps () {
     GError *error = NULL;
     gboolean ret = FALSE;
+    guint i = 0;
+    gboolean status = FALSE;
 
     ret = have_kernel_module ("zram", &error);
     if (!ret) {
@@ -72,13 +86,31 @@ gboolean bd_kbd_check_deps () {
     if (!ret)
         return FALSE;
 
+    ret = TRUE;
 #ifdef WITH_BD_BCACHE
-    ret = bd_utils_check_util_version ("make-bcache", NULL, NULL, NULL, &error);
-    if (!ret && error) {
-        g_warning("Cannot load the kbd plugin: %s" , error->message);
-        g_clear_error (&error);
-    }
+    for (i=0; i < DEPS_LAST; i++) {
+#else
+    /* we need to disable the type-limits check because GCC is too clever and
+       complains that 0 is never < 0, but DEPS_LAST may be something larger in
+       the future */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wtype-limits"
+    /* skip checking for 'make-bcache' (MUST BE LAST IN THE LIST OF DEPS!) */
+    for (i=0; i < DEPS_LAST-1; i++) {
+#pragma GCC diagnostic pop
 #endif
+        status = bd_utils_check_util_version (deps[i].name, deps[i].version,
+                                              deps[i].ver_arg, deps[i].ver_regexp, &error);
+        if (!status)
+            g_warning ("%s", error->message);
+        else
+            g_atomic_int_or (&avail_deps, 1 << i);
+        g_clear_error (&error);
+        ret = ret && status;
+    }
+
+    if (!ret)
+        g_warning("Cannot load the kbd plugin");
 
     return ret;
 }
@@ -106,6 +138,24 @@ void bd_kbd_close () {
     c_locale = (locale_t) 0;
 }
 
+/**
+ * bd_kbd_is_tech_avail:
+ * @tech: the queried tech
+ * @mode: a bit mask of queried modes of operation (#BDKBDTechMode) for @tech
+ * @error: (out): place to store error (details about why the @tech-@mode combination is not available)
+ *
+ * Returns: whether the @tech-@mode combination is available -- supported by the
+ *          plugin implementation and having all the runtime dependencies available
+ */
+gboolean bd_kbd_is_tech_avail (BDKBDTech tech, guint64 mode, GError **error) {
+    /* all combinations are supported by this implementation of the plugin, but
+       bcache creation requires the 'make-bcache' utility */
+    if (tech == BD_KBD_TECH_BCACHE && (mode & BD_KBD_TECH_MODE_CREATE))
+        return check_deps (&avail_deps, DEPS_MAKEBCACHE_MASK, deps, DEPS_LAST, &deps_check_lock, error);
+    else
+        return TRUE;
+}
+
 /**
  * bd_kbd_error_quark: (skip)
  */
@@ -135,7 +185,6 @@ void bd_kbd_zram_stats_free (BDKBDZramStats *data) {
     g_free (data);
 }
 
-#ifdef WITH_BD_BCACHE
 BDKBDBcacheStats* bd_kbd_bcache_stats_copy (BDKBDBcacheStats *data) {
     BDKBDBcacheStats *new = g_new0 (BDKBDBcacheStats, 1);
 
@@ -155,7 +204,6 @@ void bd_kbd_bcache_stats_free (BDKBDBcacheStats *data) {
     g_free (data->state);
     g_free (data);
 }
-#endif
 
 static gboolean have_kernel_module (const gchar *module_name, GError **error) {
     gint ret = 0;
@@ -306,6 +354,8 @@ static gboolean unload_kernel_module (const gchar *module_name, GError **error)
  * Returns: whether @num_devices zRAM devices were successfully created or not
  *
  * **Lengths of @size and @nstreams (if given) have to be >= @num_devices!**
+ *
+ * Tech category: %BD_KBD_TECH_ZRAM-%BD_KBD_TECH_MODE_CREATE
  */
 gboolean bd_kbd_zram_create_devices (guint64 num_devices, const guint64 *sizes, const guint64 *nstreams, GError **error) {
     gchar *opts = NULL;
@@ -389,6 +439,8 @@ gboolean bd_kbd_zram_create_devices (guint64 num_devices, const guint64 *sizes,
  * The only way how to destroy zRAM device right now is to unload the 'zram'
  * module and thus destroy all of them. That's why this function doesn't allow
  * specification of which devices should be destroyed.
+ *
+ * Tech category: %BD_KBD_TECH_ZRAM-%BD_KBD_TECH_MODE_DESTROY
  */
 gboolean bd_kbd_zram_destroy_devices (GError **error) {
     gboolean ret = FALSE;
@@ -428,6 +480,8 @@ static guint64 get_number_from_file (const gchar *path, GError **error) {
  * @error: (out): place to store error (if any)
  *
  * Returns: whether a new zRAM device was added or not
+ *
+ * Tech category: %BD_KBD_TECH_ZRAM-%BD_KBD_TECH_MODE_MODIFY
  */
 gboolean bd_kbd_zram_add_device (guint64 size, guint64 nstreams, gchar **device, GError **error) {
     gchar *path = NULL;
@@ -490,6 +544,8 @@ gboolean bd_kbd_zram_add_device (guint64 size, guint64 nstreams, gchar **device,
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the @device was successfully removed or not
+ *
+ * Tech category: %BD_KBD_TECH_ZRAM-%BD_KBD_TECH_MODE_MODIFY
  */
 gboolean bd_kbd_zram_remove_device (const gchar *device, GError **error) {
     gchar *dev_num_str = NULL;
@@ -673,6 +729,8 @@ static gboolean get_zram_stats_new (const gchar *device, BDKBDZramStats* stats,
  * @error: (out): place to store error (if any)
  *
  * Returns: (transfer full): statistics for the zRAM device
+ *
+ * Tech category: %BD_KBD_TECH_ZRAM-%BD_KBD_TECH_MODE_QUERY
  */
 BDKBDZramStats* bd_kbd_zram_get_stats (const gchar *device, GError **error) {
     gchar *path = NULL;
@@ -748,8 +806,6 @@ BDKBDZramStats* bd_kbd_zram_get_stats (const gchar *device, GError **error) {
 }
 
 
-#ifdef WITH_BD_BCACHE
-
 gboolean wait_for_file (const char *filename) {
     gint count = 500;
     while (count > 0) {
@@ -772,6 +828,8 @@ gboolean wait_for_file (const char *filename) {
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the bcache device was successfully created or not
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_CREATE
  */
 gboolean bd_kbd_bcache_create (const gchar *backing_device, const gchar *cache_device, const BDExtraArg **extra, const gchar **bcache_device, GError **error) {
     const gchar *argv[6] = {"make-bcache", "-B", backing_device, "-C", cache_device, NULL};
@@ -791,6 +849,9 @@ gboolean bd_kbd_bcache_create (const gchar *backing_device, const gchar *cache_d
     guint n = 0;
     gchar device_uuid[2][64];
 
+    if (!check_deps (&avail_deps, DEPS_MAKEBCACHE_MASK, deps, DEPS_LAST, &deps_check_lock, error))
+        return FALSE;
+
     msg = g_strdup_printf ("Started creation of bcache on '%s' and '%s'", backing_device, cache_device);
     progress_id = bd_utils_report_started (msg);
     g_free (msg);
@@ -901,6 +962,8 @@ gboolean bd_kbd_bcache_create (const gchar *backing_device, const gchar *cache_d
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the @c_set_uuid cache was successfully attached to @bcache_device or not
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_MODIFY
  */
 gboolean bd_kbd_bcache_attach (const gchar *c_set_uuid, const gchar *bcache_device, GError **error) {
     gchar *path = NULL;
@@ -936,6 +999,8 @@ gboolean bd_kbd_bcache_attach (const gchar *c_set_uuid, const gchar *bcache_devi
  * Returns: whether the bcache device @bcache_device was successfully destroyed or not
  *
  * Note: Flushes the cache first.
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_MODIFY
  */
 gboolean bd_kbd_bcache_detach (const gchar *bcache_device, gchar **c_set_uuid, GError **error) {
     gchar *path = NULL;
@@ -1025,6 +1090,8 @@ gboolean bd_kbd_bcache_detach (const gchar *bcache_device, gchar **c_set_uuid, G
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the bcache device @bcache_device was successfully destroyed or not
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_DESTROY
  */
 gboolean bd_kbd_bcache_destroy (const gchar *bcache_device, GError **error) {
     gchar *path = NULL;
@@ -1089,6 +1156,8 @@ gboolean bd_kbd_bcache_destroy (const gchar *bcache_device, GError **error) {
  * @error: (out): place to store error (if any)
  *
  * Returns: current mode of the @bcache_device
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_QUERY
  */
 BDKBDBcacheMode bd_kbd_bcache_get_mode (const gchar *bcache_device, GError **error) {
     gchar *path = NULL;
@@ -1143,6 +1212,8 @@ BDKBDBcacheMode bd_kbd_bcache_get_mode (const gchar *bcache_device, GError **err
  * @error: (out): place to store error (if any)
  *
  * Returns: (transfer none): string representation of @mode or %NULL in case of error
+ *
+ * Tech category: always available
  */
 const gchar* bd_kbd_bcache_get_mode_str (BDKBDBcacheMode mode, GError **error) {
     if (mode <= BD_KBD_MODE_UNKNOWN)
@@ -1160,6 +1231,8 @@ const gchar* bd_kbd_bcache_get_mode_str (BDKBDBcacheMode mode, GError **error) {
  * @error: (out): place to store error (if any)
  *
  * Returns: mode matching the @mode_str given or %BD_KBD_MODE_UNKNOWN in case of no match
+ *
+ * Tech category: always available
  */
 BDKBDBcacheMode bd_kbd_bcache_get_mode_from_str (const gchar *mode_str, GError **error) {
     if (g_strcmp0 (mode_str, "writethrough") == 0)
@@ -1187,6 +1260,8 @@ BDKBDBcacheMode bd_kbd_bcache_get_mode_from_str (const gchar *mode_str, GError *
  * @error: (out): place to store error (if any)
  *
  * Returns: whether the mode was successfully set or not
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_MODIFY
  */
 gboolean bd_kbd_bcache_set_mode (const gchar *bcache_device, BDKBDBcacheMode mode, GError **error) {
     gchar *path = NULL;
@@ -1299,6 +1374,8 @@ static gboolean get_cache_size_used (const gchar *cache_dev_sys, guint64 *size,
  *
  * Returns: (transfer full): status of the @bcache_device or %NULL in case of
  *                           error (@error is set)
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_QUERY
  */
 BDKBDBcacheStats* bd_kbd_bcache_status (const gchar *bcache_device, GError **error) {
     gchar *path = NULL;
@@ -1468,6 +1545,8 @@ static gchar* get_device_name (const gchar *major_minor, GError **error) {
  *
  * Note: returns the name of the first backing device of @bcache_device (in case
  *       there are more)
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_QUERY
  */
 gchar* bd_kbd_bcache_get_backing_device (const gchar *bcache_device, GError **error) {
     gchar *path = NULL;
@@ -1520,6 +1599,8 @@ gchar* bd_kbd_bcache_get_backing_device (const gchar *bcache_device, GError **er
  *
  * Note: returns the name of the first cache device of @bcache_device (in case
  *       there are more)
+ *
+ * Tech category: %BD_KBD_TECH_BCACHE-%BD_KBD_TECH_MODE_QUERY
  */
 gchar* bd_kbd_bcache_get_cache_device (const gchar *bcache_device, GError **error) {
     gchar *path = NULL;
@@ -1561,4 +1642,3 @@ gchar* bd_kbd_bcache_get_cache_device (const gchar *bcache_device, GError **erro
 
     return ret;
 }
-#endif  /* WITH_BCACHE */
