@@ -5,22 +5,34 @@ import overrides_hack
 from utils import fake_path
 
 from gi.repository import GLib, BlockDev
-if not BlockDev.is_initialized():
-    assert BlockDev.init(None, None)
+
 
 class LibraryOpsTestCase(unittest.TestCase):
     log = ""
 
+    # all plugins expcept mpath -- it doesn't have all the dependencies on Debian
+    # and we don't need it for this test
+    requested_plugins = BlockDev.plugin_specs_from_names(("btrfs", "crypto", "dm",
+                                                          "fs", "kbd", "loop", "lvm",
+                                                          "mdraid", "part", "swap"))
+
+    @classmethod
+    def setUpClass(cls):
+        if not BlockDev.is_initialized():
+            BlockDev.init(cls.requested_plugins, None)
+        else:
+            BlockDev.reinit(cls.requested_plugins, True, None)
+
     def setUp(self):
         self.addCleanup(self._clean_up)
 
     def _clean_up(self):
         # change the sources back and recompile
         os.system("sed -ri 's?1024;//test-change?BD_LVM_MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
-        os.system("make &> /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # try to get everything back to normal by (re)loading all plugins
-        BlockDev.reinit(None, True, None)
+        BlockDev.reinit(self.requested_plugins, True, None)
 
     # recompiles the LVM plugin
     @unittest.skipIf("SKIP_SLOW" in os.environ, "skipping slow tests")
@@ -33,26 +45,26 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         # change the sources and recompile
         os.system("sed -ri 's?BD_LVM_MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
-        os.system("make &> /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # library should successfully reinitialize without reloading plugins
-        self.assertTrue(BlockDev.reinit(None, False, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, False, None))
 
         # LVM plugin not reloaded, max LV size should be the same
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
 
         # library should successfully reinitialize reloading plugins
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         # LVM plugin reloaded, max LV size should be 1024 bytes
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
 
         # change the sources back and recompile
         os.system("sed -ri 's?1024;//test-change?BD_LVM_MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
-        os.system("make &> /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # library should successfully reinitialize reloading original plugins
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
     # recompiles the LVM plugin
     @unittest.skipIf("SKIP_SLOW" in os.environ, "skipping slow tests")
@@ -63,7 +75,7 @@ class LibraryOpsTestCase(unittest.TestCase):
         self.assertTrue(BlockDev.is_initialized())
 
         # init() called twice, should give a warning and return False
-        self.assertFalse(BlockDev.init(None, None))
+        self.assertFalse(BlockDev.init(self.requested_plugins, None))
 
         # max LV size should be something sane (not 1024 bytes)
         orig_max_size = BlockDev.lvm_get_max_lv_size()
@@ -71,14 +83,14 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         # change the sources and recompile
         os.system("sed -ri 's?BD_LVM_MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
-        os.system("make &>/dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # proclaim the new build a different plugin
         os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so")
 
         # change the sources back and recompile
         os.system("sed -ri 's?1024;//test-change?BD_LVM_MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
-        os.system("make &>/dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # force the new plugin to be used
         ps = BlockDev.PluginSpec()
@@ -114,22 +126,22 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         # change the sources and recompile
         os.system("sed -ri 's?BD_LVM_MAX_LV_SIZE;?1024;//test-change?' src/plugins/lvm.c > /dev/null")
-        os.system("make &>/dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # proclaim the new build a different plugin
-        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so.0")
+        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so.2")
 
         # change the sources back and recompile
         os.system("sed -ri 's?1024;//test-change?BD_LVM_MAX_LV_SIZE;?' src/plugins/lvm.c > /dev/null")
-        os.system("make &>/dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # now reinit the library with the config preferring the new build
         orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
         os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/plugin_prio_conf.d"
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         # new LVM plugin loaded, max LV size should be 1024 bytes
-        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm2.so.0")
+        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm2.so.2")
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
 
         # reinit with the original config
@@ -137,7 +149,7 @@ class LibraryOpsTestCase(unittest.TestCase):
             os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
         else:
             del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
 
@@ -145,10 +157,10 @@ class LibraryOpsTestCase(unittest.TestCase):
         # build
         orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
         os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/plugin_multi_conf.d"
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         # new LVM plugin loaded, max LV size should be 1024 bytes
-        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm2.so.0")
+        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm2.so.2")
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), 1024)
 
         # reinit with the original config
@@ -156,7 +168,7 @@ class LibraryOpsTestCase(unittest.TestCase):
             os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
         else:
             del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
 
@@ -176,24 +188,24 @@ class LibraryOpsTestCase(unittest.TestCase):
         self.assertNotEqual(orig_max_size, 1024)
 
         # change the sources and recompile
-        os.system("sed -ri 's?gboolean check\(\) \{?gboolean check() { return FALSE;//test-change?' src/plugins/lvm.c > /dev/null")
-        os.system("make &>/dev/null")
+        os.system("sed -ri 's?gboolean bd_lvm_check_deps \(\) \{?gboolean bd_lvm_check_deps () { return FALSE;//test-change?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # proclaim the new build a different plugin
-        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so.0")
+        os.system("cp src/plugins/.libs/libbd_lvm.so src/plugins/.libs/libbd_lvm2.so.2")
 
         # change the sources back and recompile
-        os.system("sed -ri 's?gboolean check\(\) \{ return FALSE;//test-change?gboolean check() {?' src/plugins/lvm.c > /dev/null")
-        os.system("make &>/dev/null")
+        os.system("sed -ri 's?gboolean bd_lvm_check_deps \(\) \{ return FALSE;//test-change?gboolean bd_lvm_check_deps () {?' src/plugins/lvm.c > /dev/null")
+        os.system("make -C src/plugins/ libbd_lvm.la >/dev/null 2>&1")
 
         # now reinit the library with the config preferring the new build
         orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
         os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/plugin_prio_conf.d"
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         # the original plugin should be loaded because the new one should fail
         # to load (due to check() returning FALSE)
-        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm.so.0")
+        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm.so.2")
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
 
         # reinit with the original config
@@ -201,7 +213,7 @@ class LibraryOpsTestCase(unittest.TestCase):
             os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
         else:
             del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
 
@@ -209,11 +221,11 @@ class LibraryOpsTestCase(unittest.TestCase):
         # build
         orig_conf_dir = os.environ.get("LIBBLOCKDEV_CONFIG_DIR")
         os.environ["LIBBLOCKDEV_CONFIG_DIR"] = "tests/plugin_multi_conf.d"
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         # the original plugin should be loaded because the new one should fail
         # to load (due to check() returning FALSE)
-        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm.so.0")
+        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.LVM), "libbd_lvm.so.2")
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
 
         # reinit with the original config
@@ -221,7 +233,7 @@ class LibraryOpsTestCase(unittest.TestCase):
             os.environ["LIBBLOCKDEV_CONFIG_DIR"] = orig_conf_dir
         else:
             del os.environ["LIBBLOCKDEV_CONFIG_DIR"]
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         self.assertEqual(BlockDev.lvm_get_max_lv_size(), orig_max_size)
 
@@ -238,13 +250,13 @@ class LibraryOpsTestCase(unittest.TestCase):
     def test_logging_setup(self):
         """Verify that setting up logging works as expected"""
 
-        self.assertTrue(BlockDev.reinit(None, False, self.my_log_func))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, False, self.my_log_func))
 
         succ = BlockDev.utils_exec_and_report_error(["true"])
         self.assertTrue(succ)
 
         # reinit with no logging function should change nothing about logging
-        self.assertTrue(BlockDev.reinit(None, False, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, False, None))
 
         succ, out = BlockDev.utils_exec_and_capture_output(["echo", "hi"])
         self.assertTrue(succ)
@@ -273,7 +285,7 @@ class LibraryOpsTestCase(unittest.TestCase):
         ps.so_name = ""
         self.assertTrue(BlockDev.reinit([ps], True, None))
         self.assertEqual(BlockDev.get_available_plugin_names(), ["btrfs"])
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
     def test_not_implemented(self):
         """Verify that unloaded/unimplemented functions report errors"""
@@ -291,7 +303,7 @@ class LibraryOpsTestCase(unittest.TestCase):
         with self.assertRaises(GLib.GError):
             BlockDev.lvm_get_max_lv_size()
 
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         # loaded again
         self.assertTrue(BlockDev.lvm_get_max_lv_size() > 0)
@@ -301,7 +313,7 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         # the library is already initialized, ensure_init() shonuld do nothing
         avail_plugs = BlockDev.get_available_plugin_names()
-        self.assertTrue(BlockDev.ensure_init(None, None))
+        self.assertTrue(BlockDev.ensure_init(self.requested_plugins, None))
         self.assertEqual(avail_plugs, BlockDev.get_available_plugin_names())
 
         # reinit with a subset of plugins
@@ -323,7 +335,7 @@ class LibraryOpsTestCase(unittest.TestCase):
         self.assertEqual(BlockDev.get_available_plugin_names(), [])
 
         # ensure_init to load all plugins back
-        self.assertTrue(BlockDev.ensure_init(None, None))
+        self.assertTrue(BlockDev.ensure_init(self.requested_plugins, None))
         self.assertGreaterEqual(len(BlockDev.get_available_plugin_names()), 9)
 
     def test_try_reinit(self):
@@ -331,18 +343,20 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         # try reinitializing with only some utilities being available and thus
         # only some plugins able to load
-        with fake_path("tests/lib_missing_utils", keep_utils=["swapon", "swapoff", "mkswap", "lvm", "btrfs"]):
-            succ, loaded = BlockDev.try_reinit(None, True, None)
+        with fake_path("tests/lib_missing_utils", keep_utils=["swapon", "swapoff", "mkswap", "lvm",
+                                                              "btrfs", "thin_metadata_size", "swaplabel"]):
+            succ, loaded = BlockDev.try_reinit(self.requested_plugins, True, None)
             self.assertFalse(succ)
             for plug_name in ("swap", "lvm", "btrfs"):
                 self.assertIn(plug_name, loaded)
 
         # reset back to all plugins
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
 
         # now the same with a subset of plugins requested
         plugins = BlockDev.plugin_specs_from_names(["btrfs", "lvm", "swap"])
-        with fake_path("tests/lib_missing_utils", keep_utils=["swapon", "swapoff", "mkswap", "lvm", "btrfs"]):
+        with fake_path("tests/lib_missing_utils", keep_utils=["swapon", "swapoff", "mkswap", "lvm", "btrfs",
+                                                              "thin_metadata_size", "swaplabel"]):
             succ, loaded = BlockDev.try_reinit(plugins, True, None)
             self.assertTrue(succ)
             self.assertEqual(set(loaded), set(["swap", "lvm", "btrfs"]))
@@ -352,9 +366,24 @@ class LibraryOpsTestCase(unittest.TestCase):
 
         orig_lang = os.environ.get("LANG")
         os.environ["LANG"] = "cs.CZ_UTF-8"
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
         if orig_lang:
             os.environ["LANG"] = orig_lang
         else:
             del os.environ["LANG"]
-        self.assertTrue(BlockDev.reinit(None, True, None))
+        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
+
+    def test_dep_checks_disabled(self):
+        """Verify that disabling runtime dep checks works"""
+
+        with fake_path(all_but="mkswap"):
+            # should fail because of 'mkswap' missing
+            with self.assertRaises(GLib.GError):
+                BlockDev.reinit(self.requested_plugins, True, None)
+
+        os.environ["LIBBLOCKDEV_SKIP_DEP_CHECKS"] = ""
+        self.addCleanup(os.environ.pop, "LIBBLOCKDEV_SKIP_DEP_CHECKS")
+
+        with fake_path(all_but="mkswap"):
+            # should load just fine, skipping the runtime dep checks
+            self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
